//--------------------------------------------------------------------------------//
//-                   HemiOs ( Hemisson Operating System )                       -//
//-																							         -//
//-  Ici un exemple avec le module camra linaire.                              -//
//-  Placer le robot dans une enceinte ferme avec des murs clairs.              -//
//-  Le robot ralise une procdure de rglage de la dure d'exposition,         -//
//-  puis cherche en tournant sur lui mme une barre sombre.                     -//
//-  Placer devant le mur une barre noire de 20mm, le robot s'arrte devant      -//
//-  et suit cette barre ...															         -//
//-																							         -//
//- DIDASTEL Provence - Octobre 2003                                             -//
//-																							         -//
//--------------------------------------------------------------------------------//

// Placer le fichier "hemlincam.h" dans le rpertoire de votre projet et ajouter
#include "hemlincam.h"
// dans le fichier "main.c" de ce projet.

// Mettre le contenu de cette fonction dans la fonction "main()" de votre projet ...
void ExempleCameraLineaire(void)
{
unsigned char DureeExpo, Pix;
unsigned char ExpoOk;
int i, DebutNoir = 0;

   // Initialisation du Hemisson
   hemisson_init();

   // On arrte ci-dessous tous les fonctionnement par dfaut inutile
   // Arrt de la lecture automatique des IRs
	hemisson_config_auto_refresh_sensors(MANUAL);
   // Arrt de la lecture automatique de la tlcommande TV
	hemisson_config_auto_refresh_tv_remote(DISABLE);
   // Arrt de la gestion par RS232 du robot
   hemisson_config_rs232_control(DISABLE);
   // Arrt de la gestion par tlcommande du robot
	hemisson_config_tv_remote_control(DISABLE);

   // Initialisation module, attente apres la mise sous tension du robot
	HemLinCam_Init();

   // On teste la presence du module US sinon on sonne 2s
   if (HemLinCam_Read_Version() != 255)
      {
      // Procedure de reglage de la dure d'exposition
      // Placer un fond clair devant la camra  10cm
      DureeExpo = 1;
      ExpoOk = 0;
      // on cherche l'exposition correcte (fonction lumire ambiante)
      while(ExpoOk < 34)
         {
         // Reglage de l'exposition
         HemLinCam_Set_Exposition_Time(DureeExpo);
         // Lecture des pixels
         HemLinCam_Read_Pixels();
         // On regarde si tous les Pixels de la zone centrale ont une dtection suffisante
         ExpoOk = 0;
         for( i = 0 ; i < 34 ; i++ )
            {
            if(HemLinCam_Pixels_Zone2[i] > 120) // Pixels de la zone centrale > 120
               ExpoOk++;
//            printf("Pixel %d : %u\r\n", i, HemLinCam_Pixels_Zone2[i]);
            }
//         printf("Duree expo : %u\r\n\n", DureeExpo);

         if(ExpoOk < 34)
            {
            // Pour sortir de la boucle de mise au point du temps d'exposition si il n'y a pas assez de lumire
            if (DureeExpo == 10)
               ExpoOk = 35;
            DureeExpo = DureeExpo + 1;
            }
        } // fin du while, recherche bonne exposition

      if ((ExpoOk>=34) && (DureeExpo < 10))  // condition de procedure de reglage dure exposition russie
         {
//         printf("Exposition OK\r\n\n");
         while(1)
            {
            // Lecture des pixels
            HemLinCam_Read_Pixels();
            DebutNoir = 0;
            // Recherche du dbut de la Zone noir
            for( i = 0 ; i < 34 ; i++ )
               {
               if (HemLinCam_Pixels_Zone2[i] < 100)
                  {
                  DebutNoir = i;
//                  printf("Pixel de debut zone noire : %d\r\n", DebutNoir);
                  break; // on sort de for
                  }
               }
            // Debut de la zone noire au centre de la camra
            if ((DebutNoir >= 12) && (DebutNoir <= 20))  // Noir au centre
               {
               hemisson_led_frontright(1);
               hemisson_led_frontleft(1);
               hemisson_set_speed(0, 0);
               }
            // Sinon on tourne en fonction de la dtection de cette zone noire
            else
               {
               hemisson_led_frontright(0);
               hemisson_led_frontleft(0);
               if (DebutNoir < 10)
                  hemisson_set_speed(-3, 3);
               else
                  hemisson_set_speed(3, -3);
               }
            }
         }
      else        // condition de procedure de reglage dure non atteinte ?
         {
//         printf("Impossible de regler la duree d'exposition ?\r\n\n");
         while(1);
         }
      }
   else  // Module non dtect ?
      {
      hemisson_beep(1);		        // Buzzer On
		hemisson_delay_s(2);			  // Attendre 2s
      hemisson_beep(0);		        // Buzzer Off
      }
}


